#include "PatternManager.h"
#include "../Lever/Lever.h"

#include "../TurnManager/TurnManager.h"

#include <rand.h>

// Current pattern
Pattern CurrentPattern = 
{
    {0, 1, 2, 3}, 0
};

/** Static Variables **/

// Called when a pattern has changed (next step or reset)
static void (*s_PatternUpdate)(Pattern* _Pattern) = NULL;

// Called when player failed to activate levers in the right order
static void (*s_PatternFailure)(Pattern* _Pattern) = NULL;

// Called when player has successfully activated levers in the right order
static void (*s_PatternSuccess)(Pattern* _Pattern) = NULL;

/** Methods **/

/**
 * Generates a new Pattern to deal with. Enjoy that, guys...
 */
Pattern* PatternManager_GeneratePattern()
{
    int i, j, temp;

    // Reset array order
    CurrentPattern.levers[0] = LEVER_A;
    CurrentPattern.levers[1] = LEVER_B;
    CurrentPattern.levers[2] = LEVER_C;
    CurrentPattern.levers[3] = LEVER_D;

    // Reset pattern step
    PatternManager_Reset();
    
    // Shuffle array
    for (i = NB_LEVERS-1; i > 0; i--)
    {
        // Pick a random index from 0 to i
        j = ((unsigned int)rand()) % (i+1);
 
        // Swap
        temp = CurrentPattern.levers[i];
        CurrentPattern.levers[i] = CurrentPattern.levers[j];
        CurrentPattern.levers[j] = temp;
    }

    return &CurrentPattern;
}

/**
 * Try to update the given pattern's state.
 * @param _Lever The lever just activated by player
 * @param _Pattern
 * @returns Returns true if the activated lever matches with the current pattern's step.
 */
UINT8 PatternManager_Next(Lever* _Lever)
{
    // If the given pattern is "updatable"
    /*
    if(!PatternManager_CanNext(CurrentPattern))
    {
        return FALSE;
    }
    */

    // If the given lever's id matches with th current pattern's step's lever to activate
    if(_Lever->id == CurrentPattern.levers[CurrentPattern.step])
    {
        // If the current step is the last one
        if(CurrentPattern.step >= (NB_LEVERS - 1))
        {
            if (s_PatternSuccess != NULL)
                (*s_PatternSuccess)(&CurrentPattern);

            TurnManager_GoToNextTurn();
        }

        // Else, if the current step is not the last one
        else
        {
            CurrentPattern.step++;
            if (s_PatternUpdate != NULL)
                (*s_PatternUpdate)(&CurrentPattern);
        }

        return TRUE;
    }

    // Else if the given lever's id matches with the current pattern's step's lever to activate
    else
    {
        CurrentPattern.step = 0;
        if (s_PatternFailure != NULL)
            (*s_PatternFailure)(&CurrentPattern);

        TurnManager_GameOver();

        return FALSE;
    }
}

/**
 * Checks if the given pattern can be "updated".
 */
UINT8 PatternManager_CanNext()
{
    return TRUE;
}

/**
 * Reset the given pattern to its initial state.
 */
void PatternManager_Reset()
{
    if(CurrentPattern.step != 0)
    {
        CurrentPattern.step = 0;
        if (s_PatternUpdate != NULL)
            (*s_PatternUpdate)(&CurrentPattern);
    }
}

/**
 * Called when a pattern has changed (next step or reset)
 */
void PatternManager_OnPatternUpdate(void (*_Callback)(Pattern*))
{
    s_PatternUpdate = _Callback;
}

/**
 * Called when player failed to activate levers in the right order.
 */
void PatternManager_OnPatternFailure(void (*_Callback)(Pattern*))
{
    s_PatternFailure = _Callback;
}

/**
 * Called when player has successfully activated levers in the right order.
 */
void PatternManager_OnPatternSuccess(void (*_Callback)(Pattern*))
{
    s_PatternSuccess = _Callback;
}