#ifndef DEF_PATTERN

#define DEF_PATTERN

#include "../Lever/Lever.h"

struct Pattern
{
    UINT8 levers[4];
    UINT8 step;
};
typedef struct Pattern Pattern;

/**
 * Generates a new Pattern to deal with. Enjoy that, guys...
 */
Pattern* PatternManager_GeneratePattern();

/**
 * Try to update the given pattern's state.
 * @param _Lever The lever just activated by player
 * @param _Pattern
 * @returns Returns true if the activated lever matches with the current pattern's step.
 */
UINT8 PatternManager_Next(Lever* _Lever);

/**
 * Checks if the given pattern can be "updated".
 */
UINT8 PatternManager_CanNext();

/**
 * Reset the given pattern to its initial state.
 */
void PatternManager_Reset();

/**
 * Called when a pattern has changed (next step or reset)
 * @param _Pattern The updated pattern AFTER it has been updated.
 */
void PatternManager_OnPatternUpdate(void (*_Callback)(Pattern* _Pattern));

/**
 * Called when player failed to activate levers in the right order.
 * @param _Pattern The updated pattern AFTER it has been reseted.
 */
void PatternManager_OnPatternFailure(void (*_Callback)(Pattern* _Pattern));

/**
 * Called when player has successfully activated levers in the right order.
 * @param _Pattern The updated pattern AFTER it has been validated.
 */
void PatternManager_OnPatternSuccess(void (*_Callback)(Pattern* _Pattern));

/** Exposed Variables **/

// Current pattern
extern Pattern CurrentPattern;

#endif